﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.Sprites;

[RequireComponent(typeof(AudioSource))]
public class Player : Actor {

	float _bob = 0f;

	bool _hurt = false;
	float _invulnerableTime = 2.0f;

	[SerializeField]
	HealthController _healthController;

	[SerializeField]
	AudioClip[] _hurtSounds;

	AudioSource _soundSource;

	[SerializeField]
	MultiplierController _multiplier;

	[SerializeField]
	BeatHit _beatHit;

	float _positionY = -2f;

	float _spriteIndex = 0;

	Vector2 _targetMotion = Vector2.zero;
	float _motionAdjustSpeed = 5f;

	float _maxSpeed = 10f;
	float _powerMaxSpeed = 15f;
	float _speed = 0f;
	float _speedFalloff = 0.01f;

	float _boundryWidth = 2.2f;


	[SerializeField]
	Sprite[] _hurtSprite;

	[SerializeField]
	Sprite[] _upSprite;
	[SerializeField]
	Sprite[] _sideUpSprite;
	[SerializeField]
	Sprite[] _sideSprite;

	[SerializeField]
	SpriteRenderer _sprite;

	[SerializeField]
	Sprite _boardUpSprite;
	[SerializeField]
	Sprite _boardRightUpSprite;
	[SerializeField]
	Sprite _boardLeftUpSprite;
	[SerializeField]
	Sprite _boardRightSprite;
	[SerializeField]
	Sprite _boardLeftSprite;

	[SerializeField]
	SpriteRenderer _boardSprite;

	[SerializeField]
	ParticleSystem _particleSystem;

	[SerializeField]
	GameObject _deadPlayer;
	[SerializeField]
	GameObject _deadBoard;

	Rigidbody2D _rigidBody;

	float _minVertSpeed = 0.5f;

	void Awake(){
		_rigidBody = GetComponent<Rigidbody2D> ();
		_soundSource = GetComponent<AudioSource> ();
	}

	void Update()
	{
		if (_healthController._health <= 0) {
			Instantiate (_deadPlayer, _sprite.transform.position, Quaternion.identity);
			Instantiate (_deadBoard, _boardSprite.transform.position, Quaternion.identity);
			_soundSource.clip = _hurtSounds [Random.Range (0, _hurtSounds.Length)];
			_soundSource.Play ();

			gameObject.SetActive(false);
		}

		_sprite.color = Color.Lerp (_sprite.color, Color.white, Time.deltaTime * 2f);
		_boardSprite.color = Color.Lerp (_sprite.color, Color.white, Time.deltaTime * 2f);
		base.Update ();
		_motion = (Vector2)Vector3.RotateTowards (_motion, _targetMotion, _motionAdjustSpeed * Time.deltaTime, 1.0f);
		if (_motion.y <= 0)
			_motion = new Vector2 (_motion.x, 0).normalized;
		_speed = Mathf.Lerp (_speed, 0f, _speedFalloff);

		_particleSystem.startLifetime = Mathf.Max(_speed / 20f, _minVertSpeed);

		Border ();

		Draw ();
	}

	void Border()
	{
		if ((_rigidBody.position.x < -_boundryWidth && _targetMotion.x < 0f)
			|| (_rigidBody.position.x > _boundryWidth && _targetMotion.x > 0f)) {
			_targetMotion = new Vector2 (_targetMotion.x * -1, _targetMotion.y).normalized;
			if (_targetMotion.y <= 0f) {
				_targetMotion = new Vector2 (_targetMotion.x, _positionY).normalized;
			}
		}
	}

	void FixedUpdate(){
		var newPos = _rigidBody.position + (_motion.normalized * _speed) * Time.deltaTime;
		newPos.y = Mathf.Lerp(newPos.y, _positionY +_speed/20f, Time.deltaTime*5f);
		_rigidBody.MovePosition(newPos);
	}

	void Draw()
	{
		_bob += Time.deltaTime * 5f;
		_sprite.transform.position = new Vector3 (_sprite.transform.position.x, _sprite.transform.position.y + (Mathf.Sin(_bob)*0.01f), 1f);
		_boardSprite.transform.position = new Vector3 (_boardSprite.transform.position.x, _boardSprite.transform.position.y + (Mathf.Sin(_bob)*0.01f), 1f);

		transform.localScale = GetScale (new Vector3 (2f, 2f, 2f));
		var rotation = Quaternion.LookRotation (Vector3.forward, _motion).eulerAngles;

		_spriteIndex += Time.deltaTime * 4f;
		if (_spriteIndex > 3f)
			_spriteIndex = 0f;

		if (_direction == Direction.Up) {
			_sprite.sprite = _upSprite[GetIndex()];
			_sprite.flipX = false;

			_boardSprite.sprite = _boardUpSprite;
		}

		if (_direction == Direction.UpRight) {
			_sprite.sprite = _sideUpSprite[GetIndex()];
			_sprite.flipX = true;

			_boardSprite.sprite = _boardRightUpSprite;
		}

		if (_direction == Direction.Right) {
			_sprite.sprite = _sideSprite[GetIndex()];
			_sprite.flipX = true;

			_boardSprite.sprite = _boardRightSprite;
		}

		if (_direction == Direction.Left) {
			_sprite.sprite = _sideSprite[GetIndex()];
			_sprite.flipX = false;

			_boardSprite.sprite = _boardLeftSprite;
		}

		if (_direction == Direction.UpLeft) {
			_sprite.sprite = _sideUpSprite[GetIndex()];
			_sprite.flipX = false;

			_boardSprite.sprite = _boardLeftUpSprite;
		}

		if (_hurt) {
			_sprite.sprite = _hurtSprite [(int)Mathf.Clamp(GetIndex (), 0f, 2f)];

			if (_direction == Direction.DownLeft || _direction == Direction.Left) {
				_sprite.flipX = false;
			} else if (_direction == Direction.DownLeft || _direction == Direction.Left) {
				_sprite.flipX = true;
			}
		}
	}

	void OnTriggerEnter2D(Collider2D collider)
	{
		if (collider.gameObject.tag == "Wave") {
			_targetMotion = collider.gameObject.GetComponent<Wave> ().GetMotion ();
			_speed = _maxSpeed;
			if (_beatHit.Active ()) {
				_sprite.color = Color.cyan;
				_boardSprite.color = Color.magenta;
				_speed = _maxSpeed * 1.5f;
				_multiplier.Hit (collider.gameObject);
				//Debug.Log ("HIT!");
			}
		} 
		if (collider.gameObject.tag == "Hurt" && !_hurt) {
			if (_hurtSounds.Length > 0) {
				_multiplier.ResetMultiplier ();
				_soundSource.clip = _hurtSounds [Random.Range (0, _hurtSounds.Length)];
				_soundSource.Play ();
				_motion = Vector2.zero;
				_speed = 0f;
				_sprite.color = Color.red;
				_boardSprite.color = Color.red;
				_hurt = true;
				Invoke ("InvulnerableOff", _invulnerableTime);
				Invoke ("Flash", _invulnerableTime/20f);
				_healthController.Hurt ();
			}
		}
	}

	void InvulnerableOff()
	{
		_hurt = false;
	}

	void Flash()
	{
		if (_hurt == true) {
			_sprite.color = Color.red;
			Invoke ("Flash2", _invulnerableTime / 20f);
		} else {
			_sprite.color = Color.white;
		}
	}

	void Flash2()
	{
		_sprite.color = Color.white;
		if (_hurt == true) {
			Invoke ("Flash", _invulnerableTime / 20f);
		}
	}

	int GetIndex()
	{
		return (int)Mathf.Round(_spriteIndex);
	}

	public Vector3 GetScale(Vector3 scale)
	{
		return Vector3.Lerp(scale, scale * (1f - (_speed/8f)), Time.deltaTime*5f);
	}

	public float GetSpeed()
	{
		return _speed/20f;
	}
}
